From 745752478c7ab895390a671ee4f7712eedf46bcc Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Mattias=20Engdeg=C3=A5rd?= <mattiase@acm.org>
Date: Thu, 2 Jul 2020 19:54:56 +0200
Subject: [PATCH] Relax portable number predicate in byte-compiler

Since Emacs has standardised on IEEE754 binary64, and the range of
integers is no longer platform-dependent, all numbers are portable.
Retain a restriction of floats to integers in [-2^53,2^53] to be
safe against x87 rounding errors (bug#42147).

* lisp/emacs-lisp/byte-opt.el (byte-opt--max-integral-float):
(byte-opt--min-integral-float):
(byte-opt--portable-max):
(byte-opt--portable-min):
(byte-opt--portable-numberp):
---
 lisp/emacs-lisp/byte-opt.el | 27 +++++++++++++--------------
 1 file changed, 13 insertions(+), 14 deletions(-)

diff --git a/lisp/emacs-lisp/byte-opt.el b/lisp/emacs-lisp/byte-opt.el
index 72c68d64b2..2c7a23ebed 100644
--- a/lisp/emacs-lisp/byte-opt.el
+++ b/lisp/emacs-lisp/byte-opt.el
@@ -672,23 +672,22 @@ byte-optimize-associative-math
 	    (apply (car form) constants))
 	form)))
 
-;; Portable Emacs integers fall in this range.
-(defconst byte-opt--portable-max #x1fffffff)
-(defconst byte-opt--portable-min (- -1 byte-opt--portable-max))
+;; Bounds of consecutive integers representable as floats.
+;; These assume IEEE 754 binary64 floats.
+(defconst byte-opt--max-integral-float (float (ash 1 53)))
+(defconst byte-opt--min-integral-float (- byte-opt--max-integral-float))
 
 ;; True if N is a number that works the same on all Emacs platforms.
-;; Portable Emacs fixnums are exactly representable as floats on all
-;; Emacs platforms, and (except for -0.0) any floating-point number
-;; that equals one of these integers must be the same on all
-;; platforms.  Although other floating-point numbers such as 0.5 are
-;; also portable, it can be tricky to characterize them portably so
-;; they are not optimized.
+;; As long as we need to support 32-bit x86 (using x87 floats), we cannot
+;; be certain that the result is portable to the last bit for all values;
+;; therefore, we only consider integral floats to be 'portable'.
 (defun byte-opt--portable-numberp (n)
-  (and (numberp n)
-       (<= byte-opt--portable-min n byte-opt--portable-max)
-       (= n (floor n))
-       (not (and (floatp n) (zerop n)
-                 (condition-case () (< (/ n) 0) (error))))))
+  (or (integerp n)
+      (and (floatp n)
+           (ignore-errors (= n (floor n)))
+           (<= byte-opt--min-integral-float
+               n
+               byte-opt--max-integral-float))))
 
 ;; Use OP to reduce any leading prefix of portable numbers in the list
 ;; (cons ACCUM ARGS) down to a single portable number, and return the
-- 
2.21.1 (Apple Git-122.3)

